@@ -25,7 +25,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * Version 1.1 20051125
+ * Version 1.4 20070904
  */
 
 #include <stdlib.h>
@@ -38,7 +38,7 @@
  * data in the sdparm_data.c tables.
  *
  * Build with a line like:
- *      gcc -Wall -o chk_sdparm_data sdparm_data.o chk_sdparm_data.c
+ *      gcc -Wall -o chk_sdparm_data sdparm_data.o sdparm_data_vendor.o chk_sdparm_data.c
  *
  */
 
@@ -90,16 +90,27 @@ static void set_cl(int off, int pdt, unsigned char mask)
     }
 }
 
-static void check(const struct sdparm_mode_page_item * mpi)
+static void check(const struct sdparm_mode_page_item * mpi,
+                  const struct sdparm_mode_page_item * mpi_b)
 {
     unsigned char mask;
     const struct sdparm_mode_page_item * kp = mpi;
     const struct sdparm_mode_page_item * jp = mpi;
     const char * acron;
     int res, prev_mp, prev_msp, prev_pdt, sbyte, sbit, nbits;
+    int second_k = 0;
+    int second_j = 0;
 
     clear_cl();
-    for (prev_mp = 0, prev_msp = 0, prev_pdt = -1; kp->acron; ++kp) {
+    for (prev_mp = 0, prev_msp = 0, prev_pdt = -1; ; ++kp) {
+        if (NULL == kp->acron) {
+            if ((NULL == mpi_b) || second_k)
+                break;
+            prev_mp = 0;
+            prev_msp = 0;
+            kp = mpi_b;
+            second_k = 1;
+        }
         acron = kp->acron ? kp->acron : "?";
         if ((prev_mp != kp->page_num) || (prev_msp != kp->subpage_num)) {
             if (prev_mp > kp->page_num)
@@ -119,10 +130,18 @@ static void check(const struct sdparm_mode_page_item * mpi)
                        prev_pdt, kp->pdt);
             prev_pdt = kp->pdt;
         }
-        for (jp = kp + 1; jp->acron; ++jp) {
-            if (0 == strcmp(acron, jp->acron))
-                printf("  acronym with this description: %s clashes with "
-                       "%s\n", kp->description, jp->description);
+        for (jp = kp + 1, second_j = second_k; ; ++jp) {
+            if (NULL == jp->acron) {
+                if ((NULL == mpi_b) || second_j)
+                    break;
+                jp = mpi_b;
+                second_j = 1;
+            }
+            if ((0 == strcmp(acron, jp->acron)) &&
+                (! (jp->flags & MF_CLASH_OK)))
+                printf("  acronym '%s' with this description: '%s'\n    "
+                       "clashes with '%s'\n", acron, kp->description,
+                       jp->description);
         }
         sbyte = kp->start_byte;
         if ((unsigned)sbyte + 8 > MAX_MP_LEN) {
@@ -202,22 +221,50 @@ static void check(const struct sdparm_mode_page_item * mpi)
     }
 }
 
+static const char * get_vendor_name(int vendor_num)
+{
+    const struct sdparm_vendor_name_t * vnp;
+
+    for (vnp = sdparm_vendor_id; vnp->acron; ++vnp) {
+        if (vendor_num == vnp->vendor_num)
+            return vnp->name;
+    }
+    return NULL;
+}
+
 int main(int argc, char ** argv)
 {
     int k;
     const struct sdparm_transport_pair * tp;
+    const struct sdparm_vendor_pair * vp;
+    const char * ccp;
 
-    printf("Check integrity of mode page item tables in sdparm\n");
+    printf("    Check integrity of mode page item tables in sdparm\n");
+    printf("    ==================================================\n\n");
     printf("Generic (i.e. non-transport specific) mode page items:\n");
-    check(sdparm_mitem_arr);
+    check(sdparm_mitem_arr, NULL);
     printf("\n");
     tp = sdparm_transport_mp;
     for (k = 0; k < 16; ++k, ++tp) {
         if (tp->mitem) {
             printf("%s mode page items:\n", sdparm_transport_id[k].name);
-            check(tp->mitem);
+            check(tp->mitem, NULL);
             printf("\n");
         }
     }
+    vp = sdparm_vendor_mp;
+    for (k = 0; k < sdparm_vendor_mp_len; ++k, ++vp) {
+        if (vp->mitem) {
+            ccp = get_vendor_name(k);
+            if (ccp)
+                printf("%s mode page items:\n", ccp);
+            else
+                printf("0x%x mode page items:\n", k);
+            check(vp->mitem, NULL);
+            printf("\n");
+        }
+    }
+    printf("Cross check Generic with SAS mode page items:\n");
+    check(sdparm_mitem_arr, sdparm_transport_mp[6].mitem);
     return 0;
 }
